#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-
""":synopsis: Standard exception classes.


Exceptions should be class objects.   The exceptions are defined in the module
:mod:`exceptions`.  This module never needs to be imported explicitly: the
exceptions are provided in the built-in namespace as well as the
:mod:`exceptions` module.

"""
